
## Instruction for Mainnet v1.2.0 Upgrade
First, go to the `iotex-var` folder on your node (where it has 3 sub-folders `etc`, `data`, `log`)
```
export IOTEX_HOME=$PWD
```
stop the running node:
```
docker stop iotex
docker rm iotex
``` 

#### Option 1 - if you are running the node WITHOUT API gateway enabled
Check files under `$IOTEX_HOME/data`, if you don't see a file named `index.db`,
then your node is **not** running as gateway. In this case, simply start the
node using image v1.2.0.
```
docker run -d --restart on-failure --name iotex \
        -p 4689:4689 \
        -p 8080:8080 \
        -v=$IOTEX_HOME/data:/var/data:rw \
        -v=$IOTEX_HOME/log:/var/log:rw \
        -v=$IOTEX_HOME/etc/config.yaml:/etc/iotex/config_override.yaml:ro \
        -v=$IOTEX_HOME/etc/genesis.yaml:/etc/iotex/genesis.yaml:ro \
        iotex/iotex-core:v1.2.0 \
        iotex-server \
        -config-path=/etc/iotex/config_override.yaml \
        -genesis-path=/etc/iotex/genesis.yaml
```
Verify the node is up and running by:
```
docker logs iotex --tail=100
```
You should see log output from the node.

**If your node is syncing blocks very slow, see [here](#slow-sync)**

#### Option 2 - if you are running the node WITH API gateway enabled
In this case, you'll need to wipe out the data and start from latest snapshot, due to index file in v1.2 is not backward-compatible.
```
rm -rf $IOTEX_HOME/data
curl -L https://t.iotex.me/mainnet-data-with-idx-latest > $IOTEX_HOME/data.tar.gz
tar -xzf data.tar.gz
```
then start the node using image v1.2.0
```
docker run -d --restart on-failure --name iotex \
        -p 4689:4689 \
        -p 14014:14014 \
        -p 8080:8080 \
        -v=$IOTEX_HOME/data:/var/data:rw \
        -v=$IOTEX_HOME/log:/var/log:rw \
        -v=$IOTEX_HOME/etc/config.yaml:/etc/iotex/config_override.yaml:ro \
        -v=$IOTEX_HOME/etc/genesis.yaml:/etc/iotex/genesis.yaml:ro \
        iotex/iotex-core:v1.2.0 \
        iotex-server \
        -config-path=/etc/iotex/config_override.yaml \
        -genesis-path=/etc/iotex/genesis.yaml \
        -plugin=gateway
```
Since downloading and extracting data may take quite some time, we **strongly**
recommend you do the upgrade in an epoch when the node is not chosen as active
delegate. This would prevent your node from being possibly probated and lose
productivity.

Verify the node is up and running by:
```
docker logs iotex --tail=100
```
You should see log output from the node.

**If your node is syncing blocks very slow, see [here](#slow-sync)**

## Instruction for Testnet v1.2.0 Upgrade
To upgrade to testnet v1.2.0, the procedure is same as that of mainnet, except these 2:

1. Download the snapshot from `https://t.iotex.me/testnet-data-with-idx-latest` if needed
2. Update the config and genesis files like below

  * Save the private key and external IP address, i.e., the `producerPrivKey:` and `externalHost` line in file `$IOTEX_HOME/etc/config.yaml`

  * Get the new config and genesis files
  ```
  curl https://raw.githubusercontent.com/iotexproject/iotex-bootstrap/v1.2.0/config_testnet.yaml > $IOTEX_HOME/etc/config.yaml
  curl https://raw.githubusercontent.com/iotexproject/iotex-bootstrap/v1.2.0/genesis_testnet.yaml > $IOTEX_HOME/etc/genesis.yaml
  ```

  * Set the private key and external IP back into `$IOTEX_HOME/etc/config.yaml`


**If your node is syncing blocks very slow, see [here](#slow-sync)**

### <a name="slow-sync"/>Troubleshoot for Slow Sync

Under certain network conditions, the node may sync slower as it should be. Follow the steps below to config it the sync mechanism to be more aggresive.

1. stop the node
```
docker stop iotex
```
2. open file `$IOTEX_HOME/etc/config.yaml`, add the following line under
`blockSync:`

![image](/changelog/sync-interval.png)

3. restart the node
```
docker restart iotex
```
