---
Postmortem ID: 1
Title: Accidentally Including Test Account Balances
Author: Zhijie Shen <zhijie@iotex.com>
Status: Mitigated
Created: 2019-04-23
Updated: 2019-04-23
---


# Abstract

When launching mainnet alpha, we created the genesis states, which have included 27 test accounts and they totally have 2.7 Billion IOTX tokens. We have burned all those tokens into the all-zero address (`io1qqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqd39ym7`) to mitigate the issue. No loss happened before the issue was mitigated.

# Cause

Software uses golang map data structure to store the initial balances in genesis, and have 27 test accounts by default. When the software loads the initial balance from the genesis yaml file, it does not overwrite the map, but append additional data to it. Therefore, these 27 test accounts which holds 2.7 Billion IOTX tokens were accidentally be set to the blockchain.

# Solution

`io1qqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqd39ym7` decodes to the all-zero address, and nobody holds the key for this address. We sentc all the balances on these 27 test accounts into this all-zero address. Therefore, 2.7 Billion additional IOTX tokens were cryptographically burned.

# Impact

The problem was identified, and mitigated before people steal tokens from the test accounts, so that there was no economic damage happened. The transfer actions that we burned these extra tokens will be permanently recorded on the blockchain.

# Activities

- 2019-04-23 08:30 PDT: the test accounts were found
- 2019-04-23 09:00 PDT: the root cause was identified
- 2019-04-23 09:10 PDT: API endpoints were shutdown and tube was shutdown
- 2019-04-23 10:00 PDT: the test accounts were audited and there was no loss
- 2019-04-23 10:20 PDT: burned all balances into the all-zero address
- 2019-04-23 11:00 PDT: verified all test account balances are cleared
- 2019-04-23 12:00 PDT: re-enable API endpoints and tube

# What Went Well

- This issue has been identified and throughly fixed within a few hours after reporting. We also validated all testing private keys within the repo and found no coins associated.
- The communication of the core members was effective.

# Follow-ups

- Bookkeep the total tokens minted on the blockchain
- Development a tool/service for monitoring the total number of coins by scanning all accounts on IoTeX blockchain
- <s>Fix the genesis initial balance data structure to overwrite instead of append</s>
https://github.com/iotexproject/iotex-core/pull/1079
- <s>Exclude `io1qqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqd39ym7` from being an action initiator</s>
https://crypto.stackexchange.com/questions/47809/why-havent-any-sha-256-collisions-been-found-yet

# Appendix
## Action hashes for burning extra coins.
```
ef8a45ee8b1044e17eb68c27f8d10040a8952ad9d181aabc9df0f6fdd84926e2
7a93542b1c6e479e6ec2e9fcf88449f2f3505effc8d06fa56cda2b718f076304
c51f6fc3df74b68f55274e12a156882e148ca4d72d5002c359e4f15ae1073e5b
7ed0639ab6e1b4d7890cd25fe5d2da511cd60f968f6dd9bc72a485b2f00037b0
d6d85c00f8566af14418e156e27595e269aa696cc8d9ccd6e45391a9a6b65169
983bcf77fd72a50f39759986138cd67a4f5a5ed5b940ccfc75c5a890247d271f
5bb51aadb65902c54c3d50d33dfca2cd17358b09cf32f7f4069a4772aceebd0b
a0e221c81e142b3072dbd83e11316477b0c7e8d440dd8ccbe144b5b5e317504e
e6a25347fbff1c0b16dd872f9df643bbf948fe96bb1a3066ef876fc5ea2558a2
41066bdf120897b6a7a154f99b8c9393240de27fcc3e5557ec4579bca0c97ea4
1b865f337341ba8f3a2eb8654af71978d9ada3625810b49136f8585aa851dd54
368a6cc7433a3c2e6c65bdadecdc1775007450a702883bb3bb8cd3c730f23d8f
2c5f02c2df6d29493eafb0854260babd99a5677f81f53553633ae3db3bb96808
7e53c002058411411ebf6035aa9b977626b7fdfad86e3df4b30025b69be6624b
cb20b5c6780dec4f0943d3fb13d43f80d99f1f0ca1e874ea299c9d8d338ceade
48ee191f856901523081fe6e8c8ab8cb2e5b5a6ebfc83f6e7de1f12846377ccf
7bb347d0d90b2daadff2c860ac60e7c381bece9effd3692d9fe2dc2bd6e7889d
c6f2e9258ec6b4d2662a8cbfb51eff4bf03da9cbd514cd8d8a70150c8ffe572c
4dd6fdd47225d75fb5d5d2088939b1d773b41ffdc76c9119b0677f2491030ef4
10341506b6a783ff5e4785ae743c5d2a0565ac61f0328fb30ddcc29adfbd594c
86862ccc20942fcc3dbc89aa80b8fba61df6170bba236105982bb4acfaf558ca
7ea37b5c6b06ea6ae7afd9a936bbeede8a298bd061c9f463a2e4fcb943dd644f
65c97b6b5be86460a19e2a43119593aa07ca700f31bd95fba8018367b58c1403
139b584ff000003d0f672379fbc8981cb86adf8ab44452f7c6e27f329a0c0ddf
f3e3d2cb9b059a0e6fdd772b5e0a877493287a139a4b7cf292d5db536c5d2832
2817777d144239ee42de814ce7742285c30fc2b77f09ae94ae280461e407fefc
6614c4a6ec06feb6b8c4a67ca5970c6cf92159cc999dbba5ee6ee11a51ea8863
9ad79279f6e9c63a877c392f697bb232c84999b567015be8d0c8cad4e0905f1c
```
