---
Postmortem ID: 2
Title: ERC20-to-native tube: 106 swaps being mistakenly re-issued
Author: Dustin <dustin.xie@iotex.io>
Created: 2019-06-25
Updated: 2019-06-25
---

# Abstract

106 ERC20-to-Native swaps are sent to V2 contract and correctly serviced. However, these are mistakenly treated as swaps sent to V1 (direct transfer to safe address *0xb9071b7016d29b2672e3796a4c4f41838cbb6fc3*) and manually re-issued again.

# Impact

A total of 1029394.54900444 IOTX native tokens have been issued to 78 addresses. Details of these 106 swap transaction on Ethereum: http://api.etherscan.io/api?module=account&action=tokentx&address=0xb9071b7016d29b2672e3796a4c4f41838cbb6fc3&startblock=7986997&endblock=8016481&sort=asc

# Root-cause

V1 tube works by monitoring IOTX ERC20 token transfer to safe address *0xb9071b7016d29b2672e3796a4c4f41838cbb6fc3*, and a transfer of equal amount of IOTX native token is triggered for each new transfer.

V2 tube records transfer in a smart contract, the service keeps pulling tranfser record by reading the contract. The safe address of V2 is set to the same address *0xb9071b7016d29b2672e3796a4c4f41838cbb6fc3*

When reviewing the 106 transfers, Etherscan API is used to pull the data and convert to CSV file for easy review. Etherscan API used our ERC20 token contract *0x6fb3e0a217407efff7ca062d46c26e5d60a14d69* in the data field "contractAddress", causing us to mistaken them as direct transfer sent to V1, while in fact they are sent to new V2 service. 

# Remedies

1. V1 tube service has been turned off. We'll only review transaction upon user report.
2. We'll require a second person review before issuing manual swap.
3. The V2 service has been re-deployed with 78 addresses (and re-issue amount) added to a "credit list". Request from this list will be held off for manual review, and only amount in excess of the credit amount will be issued. 

# Timeline

- 2019-06-18 05:52 PDT: received report that a user sent 100 IOTX-E to V1 and received 300 IOTX native token
- 2019-06-18 11:36 PDT: after reviewing all V1 swap records, 2 extra issues have been identified: 
> fe7bcb3676aabe9a6b39cb23f3a5fa41eed7ad1b: 214 native tokens
> 4fd10cc0cd952b53fbaacd11c7da9733289b3f1d：200 native tokens
- 2019-06-18 12:40 PDT: code change deployed to V1 service to disable monitoring new transfer, this effectively turned off V1 service since no new record will be added to the swap DB
- 2019-06-18 19:53 PDT: reviewed history of safe address *0xb9071b7016d29b2672e3796a4c4f41838cbb6fc3*. 6 new transfers are verified, manually added to DB, and issued
- 2019-06-19 20:00 PDT: announced new V2 service and contract address to community 
- 2019-06-23 18:47 PDT: reviewed history of safe address again, 106 new transfers are identified, manaully added to DB, and mistakenly issued

# What could be done better

- If the history review was done earlier like on 06-20, then we would end up with less than 106 new transfers. There will still be loss but it would be much smaller. **Small frequent checkup is better than big infrequent batch job, in terms of earlier issue detection and lower the impact and loss**   
- Should ask for another team member review when transactions of significant amount/value are involved   

# Follow-ups

- We should consider changing the safe address of V2 tube contract. This would eradicate the issue from happening b/c V2 transfer won't be appear in V1's safe address history 
- Development monitoring tool/service to keep track of the swap amount/activity on tube and raise alert if any suspicious/concerning transaction
